@@ -1,26 +1,15 @@
-if __name__ == '__main__':
-    import sys
-    import os
-    pkg_dir = os.path.split(os.path.abspath(__file__))[0]
-    parent_dir, pkg_name = os.path.split(pkg_dir)
-    is_pygame_pkg = (pkg_name == 'tests' and
-                     os.path.split(parent_dir)[1] == 'pygame')
-    if not is_pygame_pkg:
-        sys.path.insert(0, parent_dir)
-else:
-    is_pygame_pkg = __name__.startswith('pygame.tests.')
+# -*- coding: utf-8 -*-
 
-import unittest
-if is_pygame_pkg:
-    from pygame.tests.test_utils import example_path, png
-else:
-    from test.test_utils import example_path, png
-import pygame, pygame.image, pygame.pkgdata
-from pygame.compat import xrange_, ord_
-
-import os
 import array
+import os
 import tempfile
+import unittest
+import glob
+
+from pygame.tests.test_utils import example_path, png
+import pygame, pygame.image, pygame.pkgdata
+from pygame.compat import xrange_, ord_, unicode_
+
 
 def test_magic(f, magic_hex):
     """ tests a given file to see if the magic hex matches.
@@ -33,7 +22,7 @@ def test_magic(f, magic_hex):
     for i in range(len(magic_hex)):
         if magic_hex[i] != ord_(data[i]):
             return 0
-    
+
     return 1
 
 
@@ -62,40 +51,29 @@ class ImageModuleTest( unittest.TestCase ):
                        bluish_pixel + greyish_pixel]
 
         f_descriptor, f_path = tempfile.mkstemp(suffix='.png')
-        f = os.fdopen(f_descriptor, 'wb')
-        w = png.Writer(2, 2, alpha=True)
-        w.write(f, pixel_array)
-        f.close()
+
+        with os.fdopen(f_descriptor, 'wb') as f:
+            w = png.Writer(2, 2, alpha=True)
+            w.write(f, pixel_array)
 
         # Read the PNG file and verify that pygame interprets it correctly
         surf = pygame.image.load(f_path)
 
-        pixel_x0_y0 = surf.get_at((0, 0))
-        pixel_x1_y0 = surf.get_at((1, 0))
-        pixel_x0_y1 = surf.get_at((0, 1))
-        pixel_x1_y1 = surf.get_at((1, 1))
-
-        self.assertEquals(pixel_x0_y0, reddish_pixel)
-        self.assertEquals(pixel_x1_y0, greenish_pixel)
-        self.assertEquals(pixel_x0_y1, bluish_pixel)
-        self.assertEquals(pixel_x1_y1, greyish_pixel)
+        self.assertEqual(surf.get_at((0, 0)), reddish_pixel)
+        self.assertEqual(surf.get_at((1, 0)), greenish_pixel)
+        self.assertEqual(surf.get_at((0, 1)), bluish_pixel)
+        self.assertEqual(surf.get_at((1, 1)), greyish_pixel)
 
         # Read the PNG file obj. and verify that pygame interprets it correctly
-        f = open(f_path, 'rb')
-        surf = pygame.image.load(f)
-        f.close()
+        with open(f_path, 'rb') as f:
+            surf = pygame.image.load(f)
 
-        pixel_x0_y0 = surf.get_at((0, 0))
-        pixel_x1_y0 = surf.get_at((1, 0))
-        pixel_x0_y1 = surf.get_at((0, 1))
-        pixel_x1_y1 = surf.get_at((1, 1))
+        self.assertEqual(surf.get_at((0, 0)), reddish_pixel)
+        self.assertEqual(surf.get_at((1, 0)), greenish_pixel)
+        self.assertEqual(surf.get_at((0, 1)), bluish_pixel)
+        self.assertEqual(surf.get_at((1, 1)), greyish_pixel)
 
-        self.assertEquals(pixel_x0_y0, reddish_pixel)
-        self.assertEquals(pixel_x1_y0, greenish_pixel)
-        self.assertEquals(pixel_x0_y1, bluish_pixel)
-        self.assertEquals(pixel_x1_y1, greyish_pixel)
-
-        os.remove(f_path) 
+        os.remove(f_path)
 
     def testLoadJPG(self):
         """ see if we can load a jpg.
@@ -105,13 +83,12 @@ class ImageModuleTest( unittest.TestCase ):
         # f = os.path.join("examples", "data", "alien1.jpg")
         surf = pygame.image.load(f)
 
-        f = open(f, "rb")
+        with open(f, "rb") as f:
+            surf = pygame.image.load(f)
 
-        # f = open(os.path.join("examples", "data", "alien1.jpg"), "rb")
-        
-        surf = pygame.image.load(f)
-        
-        # surf = pygame.image.load(open(os.path.join("examples", "data", "alien1.jpg"), "rb"))
+        # with open(os.path.join("examples", "data", "alien1.jpg"), "rb") as f:
+        #     surf = pygame.image.load(open(os.path.join("examples", "data",
+        #         "alien1.jpg"), "rb"))
 
     def testSaveJPG(self):
         """ JPG equivalent to issue #211 - color channel swapping
@@ -121,7 +98,7 @@ class ImageModuleTest( unittest.TestCase ):
         """
 
         from pygame import Color, Rect
-        
+
         # The source image is a 2 by 2 square of four colors. Since JPEG is
         # lossy, there can be color bleed. Make each color square 16 by 16,
         # to avoid the significantly color value distorts found at color
@@ -163,7 +140,7 @@ class ImageModuleTest( unittest.TestCase ):
         for rect, color in squares:
             posn = rect.move((offset, offset)).topleft
             self.assertEqual(approx(jpg_surf.get_at(posn)), approx(color))
-        
+
     def testSavePNG32(self):
         """ see if we can save a png with color values in the proper channels.
         """
@@ -182,18 +159,23 @@ class ImageModuleTest( unittest.TestCase ):
         f_path = tempfile.mktemp(suffix='.png')
         pygame.image.save(surf, f_path)
 
-        # Read the PNG file and verify that pygame saved it correctly
-        width, height, pixels, metadata = png.Reader(filename=f_path).asRGBA8()
-        pixels_as_tuples = []
-        for pixel in pixels:
-            pixels_as_tuples.append(tuple(pixel))
+        try:
+            # Read the PNG file and verify that pygame saved it correctly
+            reader = png.Reader(filename=f_path)
+            width, height, pixels, metadata = reader.asRGBA8()
 
-        self.assertEquals(pixels_as_tuples[0], reddish_pixel)
-        self.assertEquals(pixels_as_tuples[1], greenish_pixel)
-        self.assertEquals(pixels_as_tuples[2], bluish_pixel)
-        self.assertEquals(pixels_as_tuples[3], greyish_pixel)
+            # pixels is a generator
+            self.assertEqual(tuple(next(pixels)), reddish_pixel)
+            self.assertEqual(tuple(next(pixels)), greenish_pixel)
+            self.assertEqual(tuple(next(pixels)), bluish_pixel)
+            self.assertEqual(tuple(next(pixels)), greyish_pixel)
 
-        os.remove(f_path)
+        finally:
+            # Ensures proper clean up.
+            if not reader.file.closed:
+                reader.file.close()
+            del reader
+            os.remove(f_path)
 
     def testSavePNG24(self):
         """ see if we can save a png with color values in the proper channels.
@@ -213,18 +195,23 @@ class ImageModuleTest( unittest.TestCase ):
         f_path = tempfile.mktemp(suffix='.png')
         pygame.image.save(surf, f_path)
 
-        # Read the PNG file and verify that pygame saved it correctly
-        width, height, pixels, metadata = png.Reader(filename=f_path).asRGB8()
-        pixels_as_tuples = []
-        for pixel in pixels:
-            pixels_as_tuples.append(tuple(pixel))
+        try:
+            # Read the PNG file and verify that pygame saved it correctly
+            reader = png.Reader(filename=f_path)
+            width, height, pixels, metadata = reader.asRGB8()
 
-        self.assertEquals(pixels_as_tuples[0], reddish_pixel)
-        self.assertEquals(pixels_as_tuples[1], greenish_pixel)
-        self.assertEquals(pixels_as_tuples[2], bluish_pixel)
-        self.assertEquals(pixels_as_tuples[3], greyish_pixel)
+            # pixels is a generator
+            self.assertEqual(tuple(next(pixels)), reddish_pixel)
+            self.assertEqual(tuple(next(pixels)), greenish_pixel)
+            self.assertEqual(tuple(next(pixels)), bluish_pixel)
+            self.assertEqual(tuple(next(pixels)), greyish_pixel)
 
-        os.remove(f_path)
+        finally:
+            # Ensures proper clean up.
+            if not reader.file.closed:
+                reader.file.close()
+            del reader
+            os.remove(f_path)
 
     def test_save(self):
 
@@ -233,11 +220,11 @@ class ImageModuleTest( unittest.TestCase ):
         magic_hex = {}
         magic_hex['jpg'] = [0xff, 0xd8, 0xff, 0xe0]
         magic_hex['png'] = [0x89 ,0x50 ,0x4e ,0x47]
-        magic_hex['tga'] = [0x0, 0x0, 0xa]
+        # magic_hex['tga'] = [0x0, 0x0, 0xa]
         magic_hex['bmp'] = [0x42, 0x4d]
 
 
-        formats = ["jpg", "png", "tga", "bmp"]
+        formats = ["jpg", "png", "bmp"]
         # uppercase too... JPG
         formats = formats + [x.upper() for x in formats]
 
@@ -245,21 +232,24 @@ class ImageModuleTest( unittest.TestCase ):
             try:
                 temp_filename = "%s.%s" % ("tmpimg", fmt)
                 pygame.image.save(s, temp_filename)
-                # test the magic numbers at the start of the file to ensure they are saved 
-                #   as the correct file type.
-                self.assertEqual((1, fmt), (test_magic(open(temp_filename, "rb"), magic_hex[fmt.lower()]), fmt))
-                # load the file to make sure it was saved correctly.  
+
+                # Using 'with' ensures the file is closed even if test fails.
+                with open(temp_filename, "rb") as handle:
+                    # Test the magic numbers at the start of the file to ensure
+                    # they are saved as the correct file type.
+                    self.assertEqual((1, fmt), (test_magic(handle,
+                        magic_hex[fmt.lower()]), fmt))
+
+                # load the file to make sure it was saved correctly.
                 #    Note load can load a jpg saved with a .png file name.
                 s2 = pygame.image.load(temp_filename)
-                #compare contents, might only work reliably for png... 
+                #compare contents, might only work reliably for png...
                 #   but because it's all one color it seems to work with jpg.
-                self.assertEquals(s2.get_at((0,0)), s.get_at((0,0)))
+                self.assertEqual(s2.get_at((0,0)), s.get_at((0,0)))
             finally:
                 #clean up the temp file, comment out to leave tmp file after run.
                 os.remove(temp_filename)
-                pass
 
-                
     def test_save_colorkey(self):
         """ make sure the color key is not changed when saving.
         """
@@ -268,7 +258,7 @@ class ImageModuleTest( unittest.TestCase ):
         s.set_colorkey((0,0,0))
         colorkey1 = s.get_colorkey()
         p1 = s.get_at((0,0))
-        
+
         temp_filename = "tmpimg.png"
         try:
             pygame.image.save(s, temp_filename)
@@ -276,15 +266,46 @@ class ImageModuleTest( unittest.TestCase ):
         finally:
             os.remove(temp_filename)
 
-
         colorkey2 = s.get_colorkey()
         # check that the pixel and the colorkey is correct.
         self.assertEqual(colorkey1, colorkey2)
         self.assertEqual(p1, s2.get_at((0,0)))
-        
-        
-        
-        
+
+    def test_load_unicode_path(self):
+        import shutil
+        orig = unicode_(example_path("data/asprite.bmp"))
+        temp = os.path.join(unicode_(example_path('data')), u'你好.bmp')
+        shutil.copy(orig, temp)
+        try:
+            im = pygame.image.load(temp)
+        finally:
+            os.remove(temp)
+
+    def _unicode_save(self, temp_file):
+        im = pygame.Surface((10, 10), 0, 32)
+        try:
+            with open(temp_file, 'w') as f:
+                pass
+            os.remove(temp_file)
+        except IOError:
+            raise unittest.SkipTest('the path cannot be opened')
+
+        self.assertFalse(os.path.exists(temp_file))
+
+        try:
+            pygame.image.save(im, temp_file)
+
+            self.assertGreater(os.path.getsize(temp_file), 10)
+        finally:
+            try:
+                os.remove(temp_file)
+            except EnvironmentError:
+                pass
+
+    def test_save_unicode_path(self):
+        """save unicode object with non-ASCII chars"""
+        self._unicode_save(u"你好.bmp")
+
     def assertPremultipliedAreEqual(self, string1, string2, source_string):
         self.assertEqual(len(string1), len(string2))
         block_size = 20
@@ -297,7 +318,7 @@ class ImageModuleTest( unittest.TestCase ):
                     source_block = source_string[block_start:block_end]
                     msg = "string difference in %d to %d of %d:\n%s\n%s\nsource:\n%s" % (block_start, block_end, len(string1), block1.encode("hex"), block2.encode("hex"), source_block.encode("hex"))
                     self.fail(msg)
-        
+
     def test_to_string__premultiplied(self):
         """ test to make sure we can export a surface to a premultiplied alpha string
         """
@@ -311,7 +332,7 @@ class ImageModuleTest( unittest.TestCase ):
                                      color[2]*color[3]/255,
                                      color[3])
                     surface_to_modify.set_at((x, y), premult_color)
-            
+
         test_surface = pygame.Surface((256, 256), pygame.SRCALPHA, 32)
         for x in xrange_(test_surface.get_width()):
             for y in xrange_(test_surface.get_height()):
@@ -325,23 +346,36 @@ class ImageModuleTest( unittest.TestCase ):
         self.assertPremultipliedAreEqual(pygame.image.tostring(test_surface, "ARGB_PREMULT"),
                                          pygame.image.tostring(premultiplied_copy, "ARGB"),
                                          pygame.image.tostring(test_surface, "ARGB"))
-        
+
         no_alpha_surface = pygame.Surface((256, 256), 0, 24)
         self.assertRaises(ValueError, pygame.image.tostring, no_alpha_surface, "RGBA_PREMULT")
-        
+
+    # Custom assert method to check for identical surfaces.
+    def _assertSurfaceEqual(self, surf_a, surf_b, msg=None):
+        a_width, a_height = surf_a.get_width(), surf_a.get_height()
+
+        # Check a few things to see if the surfaces are equal.
+        self.assertEqual(a_width, surf_b.get_width(), msg)
+        self.assertEqual(a_height, surf_b.get_height(), msg)
+        self.assertEqual(surf_a.get_size(), surf_b.get_size(), msg)
+        self.assertEqual(surf_a.get_rect(), surf_b.get_rect(), msg)
+        self.assertEqual(surf_a.get_colorkey(), surf_b.get_colorkey(), msg)
+        self.assertEqual(surf_a.get_alpha(), surf_b.get_alpha(), msg)
+        self.assertEqual(surf_a.get_flags(), surf_b.get_flags(), msg)
+        self.assertEqual(surf_a.get_bitsize(), surf_b.get_bitsize(), msg)
+        self.assertEqual(surf_a.get_bytesize(), surf_b.get_bytesize(), msg)
+        # Anything else?
+
+        # Making the method lookups local for a possible speed up.
+        surf_a_get_at = surf_a.get_at
+        surf_b_get_at = surf_b.get_at
+        for y in xrange_(a_height):
+            for x in xrange_(a_width):
+                self.assertEqual(surf_a_get_at((x, y)), surf_b_get_at((x, y)),
+                                 msg)
 
     def test_fromstring__and_tostring(self):
-        """ see if fromstring, and tostring methods are symmetric.
-        """
-        
-        def AreSurfacesIdentical(surf_a, surf_b):
-            if surf_a.get_width() != surf_b.get_width() or surf_a.get_height() != surf_b.get_height():
-                return False
-            for y in xrange_(surf_a.get_height()):
-                for x in xrange_(surf_b.get_width()):
-                    if surf_a.get_at((x,y)) != surf_b.get_at((x,y)):
-                        return False
-            return True
+        """Ensure methods tostring() and fromstring() are symmetric."""
 
         ####################################################################
         def RotateRGBAtoARGB(str_buf):
@@ -366,9 +400,10 @@ class ImageModuleTest( unittest.TestCase ):
                 byte_buf[i*4 + 2] = byte_buf[i*4 + 3]
                 byte_buf[i*4 + 3] = alpha
             return byte_buf.tostring()
-                
+
         ####################################################################
-        test_surface = pygame.Surface((64, 256), flags=pygame.SRCALPHA, depth=32)
+        test_surface = pygame.Surface((64, 256), flags=pygame.SRCALPHA,
+                                      depth=32)
         for i in xrange_(256):
             for j in xrange_(16):
                 intensity = j*16 + 15
@@ -376,36 +411,42 @@ class ImageModuleTest( unittest.TestCase ):
                 test_surface.set_at((j + 16, i), (i, intensity, i, i))
                 test_surface.set_at((j + 32, i), (i, i, intensity, i))
                 test_surface.set_at((j + 32, i), (i, i, i, intensity))
-            
-        self.assert_(AreSurfacesIdentical(test_surface, test_surface))
+
+        self._assertSurfaceEqual(test_surface, test_surface,
+                                 'failing with identical surfaces')
 
         rgba_buf = pygame.image.tostring(test_surface, "RGBA")
         rgba_buf = RotateARGBtoRGBA(RotateRGBAtoARGB(rgba_buf))
-        test_rotate_functions = pygame.image.fromstring(rgba_buf, test_surface.get_size(), "RGBA")
+        test_rotate_functions = pygame.image.fromstring(
+            rgba_buf, test_surface.get_size(), "RGBA")
 
-        self.assert_(AreSurfacesIdentical(test_surface, test_rotate_functions))
+        self._assertSurfaceEqual(test_surface, test_rotate_functions,
+                                 'rotate functions are not symmetric')
 
         rgba_buf = pygame.image.tostring(test_surface, "RGBA")
         argb_buf = RotateRGBAtoARGB(rgba_buf)
-        test_from_argb_string = pygame.image.fromstring(argb_buf, test_surface.get_size(), "ARGB")
-
-        self.assert_(AreSurfacesIdentical(test_surface, test_from_argb_string))
-        #"ERROR: image.fromstring with ARGB failed"
+        test_from_argb_string = pygame.image.fromstring(
+            argb_buf, test_surface.get_size(), "ARGB")
 
+        self._assertSurfaceEqual(test_surface, test_from_argb_string,
+                                 '"RGBA" rotated to "ARGB" failed')
 
         argb_buf = pygame.image.tostring(test_surface, "ARGB")
         rgba_buf = RotateARGBtoRGBA(argb_buf)
-        test_to_argb_string = pygame.image.fromstring(rgba_buf, test_surface.get_size(), "RGBA")
+        test_to_argb_string = pygame.image.fromstring(
+            rgba_buf, test_surface.get_size(), "RGBA")
 
-        self.assert_(AreSurfacesIdentical(test_surface, test_to_argb_string))
-        #"ERROR: image.tostring with ARGB failed"
+        self._assertSurfaceEqual(test_surface, test_to_argb_string,
+                                 '"ARGB" rotated to "RGBA" failed')
 
+        for fmt in ('ARGB', 'RGBA'):
+            fmt_buf = pygame.image.tostring(test_surface, fmt)
+            test_to_from_fmt_string = pygame.image.fromstring(
+                fmt_buf, test_surface.get_size(), fmt)
 
-        argb_buf = pygame.image.tostring(test_surface, "ARGB")
-        test_to_from_argb_string = pygame.image.fromstring(argb_buf, test_surface.get_size(), "ARGB")
-
-        self.assert_(AreSurfacesIdentical(test_surface, test_to_from_argb_string))
-        #"ERROR: image.fromstring and image.tostring with ARGB are not symmetric"
+            self._assertSurfaceEqual(test_surface, test_to_from_fmt_string,
+                                     'tostring/fromstring functions are not '
+                                     'symmetric with "{}" format'.format(fmt))
 
     def todo_test_frombuffer(self):
 
@@ -413,16 +454,16 @@ class ImageModuleTest( unittest.TestCase ):
 
           # pygame.image.frombuffer(string, size, format): return Surface
           # create a new Surface that shares data inside a string buffer
-          # 
+          #
           # Create a new Surface that shares pixel data directly from the string
           # buffer. This method takes the same arguments as
           # pygame.image.fromstring(), but is unable to vertically flip the
           # source data.
-          # 
+          #
           # This will run much faster than pygame.image.fromstring, since no
           # pixel data must be allocated and copied.
 
-        self.fail() 
+        self.fail()
 
     def todo_test_get_extended(self):
 
@@ -430,12 +471,12 @@ class ImageModuleTest( unittest.TestCase ):
 
           # pygame.image.get_extended(): return bool
           # test if extended image formats can be loaded
-          # 
+          #
           # If pygame is built with extended image formats this function will
           # return True. It is still not possible to determine which formats
           # will be available, but generally you will be able to load them all.
 
-        self.fail() 
+        self.fail()
 
     def todo_test_load_basic(self):
 
@@ -445,7 +486,7 @@ class ImageModuleTest( unittest.TestCase ):
           # pygame.image.load(fileobj, namehint=): return Surface
           # load new image from a file
 
-        self.fail() 
+        self.fail()
 
     def todo_test_load_extended(self):
 
@@ -453,7 +494,7 @@ class ImageModuleTest( unittest.TestCase ):
 
           # pygame module for image transfer
 
-        self.fail() 
+        self.fail()
 
     def todo_test_save_extended(self):
 
@@ -461,7 +502,26 @@ class ImageModuleTest( unittest.TestCase ):
 
           # pygame module for image transfer
 
-        self.fail() 
+        self.fail()
+
+    def threads_load(self, images):
+        import pygame.threads
+        for i in range(10):
+            surfs = pygame.threads.tmap(pygame.image.load, images)
+            for s in surfs:
+                self.assertIsInstance(s, pygame.Surface)
+
+    def test_load_png_threads(self):
+        self.threads_load(glob.glob(example_path("data/*.png")))
+
+    def test_load_jpg_threads(self):
+        self.threads_load(glob.glob(example_path("data/*.jpg")))
+
+    def test_load_bmp_threads(self):
+        self.threads_load(glob.glob(example_path("data/*.bmp")))
+
+    def test_load_gif_threads(self):
+        self.threads_load(glob.glob(example_path("data/*.gif")))
 
 if __name__ == '__main__':
     unittest.main()
